; 
;                                                                          
;       Read INI file and build DYNARRAY(s)                                
;       February, 14th 1993                                                
;       Harald Pitro, CompuServe ID: 100024,2662                           
;                                                                          
;       Description of INI-file:                                           
;              [Group]        --> DYNARRAY Group                           
;              Item=String    --> Group["Item"]=String                     
;              String         --> Group["Group"]=String ; PDOXWORK.INI     
;                                                                          
;              ; this is a comment, ';' must stand at start of line        
;                                                                          
;       Call:                                                              
; 		ReadIni(IniName, GroupName, ItemName)                       
;              IniName   - full name of the INI file                       
;              GroupName - name of selected group, get all if empty        
;              ItemName  - name of selected item, get all if empty         
;                                                                          
;       Return:                                                            
;              False - error while reading INI file                        
;              True  - INI was correct readed                              
;                      DYNARRAY dyGroups holds all found group names       
;                                                                          
;                      example: dyGroups["Group"] = False --> no items     
;                               dyGroups["Group"] = True  --> items readed 
;                      Use FOREACH to get all readed groups                
;                      Call ReadIni(sIni, "", " ") to get only the groups  
;                                                                          
; 


; ͻ
;      Define some needed procedures                                     
; ͼ

; Ŀ
;    Search string for substring and print an error message   
; 
PROC	SearchString(sString)
	m = SEARCH(sString, sLine)
	IF  m = 0  THEN
		MESSAGE "Bad line "+STRVAL(iLine)+" in INI file: "+sLine
		SLEEP 2000
		RETURN False
	ENDIF
	RETURN True
ENDPROC

; Ŀ
;    Skip leading spaces                                      
; 
PROC	SkipSpaces(sInp)
	PRIVATE	sInp, sOut, n, c

	n = 1
	c = SUBSTR(sInp, 1, 1)
	WHILE c = " "  OR  c = "\t"
		n = n + 1
		c = SUBSTR(sInp, n, 1)
	ENDWHILE

	RETURN SUBSTR(sInp, n, LEN(sInp)+1-n)
ENDPROC

; Ŀ
;    Double Backslashs in strings                             
; 
PROC  DoubleBackslash(sInp)
	PRIVATE	sInp, sOut, m, n

	n = 0
	sOut = ""
	WHILE True
		m = n + 1
		n = SEARCHFROM("\\", sInp, m)
		IF  n = 0  THEN
			sOut = sOut + SUBSTR(sInp, m, LEN(sInp)+1-m)
			QUITLOOP
		ENDIF

		sOut = sOut + SUBSTR(sInp, m, n-m) + "\\\\"
	ENDWHILE

	RETURN sOut
ENDPROC


; ͻ
;      Read the INI file            	                                  
; ͼ
PROC	ReadIni(sIniName, sGroupName, sItemName)
	PRIVATE	sIniName, sGroupName, sItemName, sInput,
		bGroup, bDefine, sGroup, sItem, sItemText, 
		sLine, m, n, iLine, c

	IF  ISASSIGNED(dyGroups)  THEN
		RELEASE VARS	dyGroups	; clear old results
	ENDIF
	DYNARRAY dyGroups[]			; define array for results

	IF  NOT ISFILE(sIniName)  THEN
		MESSAGE	"INI file "+sIniName+" not found."
		SLEEP 1000
		RETURN FALSE
	ENDIF

	FILEREAD sIniName TO sInput

	sGroup  = ""
	n     = 0
	iLine = 0

	WHILE True
;       Ŀ
;          Search next line, skip if line empty or comment          
;       
		m = n + 1
		n = SEARCHFROM("\n", sInput, m)
		IF  n = 0  THEN
			QUITLOOP
		ENDIF

		sLine = SkipSpaces(SUBSTR(sInput, m, n-m))
		iLine = iLine + 1
		IF  sLine = ""  OR  SUBSTR(sLine, 1, 1) = ";"  THEN
			LOOP
		ENDIF

; 	Ŀ
; 	   Does line contain an new "Group" ? If so define it.      
;       
		IF	SUBSTR(sLine, 1, 1) = "["  THEN
			IF  NOT SearchString("]")  THEN
				RETURN False
			ENDIF

			sGroup = SUBSTR(sLine, 2, m-2)
			IF  sGroupName <> ""  AND				; is group selected ?
				UPPER(sGroupName) <> UPPER(sGroup)  THEN
				bGroup = False
			ELSE
				bGroup  = True
				bDefine = False
				dyGroups[sGroup] = False
			ENDIF
			LOOP
		ENDIF

; 	Ŀ
; 	   Line must be an "Item"-line                              
;       
		IF  NOT bGroup  THEN		; skip all not needed items
			LOOP
		ENDIF

                m = SEARCH("=", sLine)          ; "Key=..." ?
	        IF  m = 0  THEN                 ; perhaps an incorrect line
                        sItem = sGroup          ; define line as it should be
                ELSE
		        sItem = SUBSTR(sLine, 1, m-1)
                ENDIF

		IF  sItemName <> ""  AND         ; skip if item isn't selected
			UPPER(sItemName) <> UPPER(sItem)  THEN
			LOOP
		ENDIF

; 	Ŀ
; 	   Define DYNARRAY and item                                 
;       
		IF  NOT dyGroups[sGroup]  THEN	; define array only if used
			EXECUTE "DYNARRAY "+sGroup+"[]\n"
			dyGroups[sGroup] = True
		ENDIF

		sItemText = SkipSpaces(SUBSTR(sLine, m+1, LEN(sLine)-m))
		IF  SUBSTR(sItemText,1,1) = "\""  THEN
			IF  SEARCHFROM("\"", sItemText, 2) > 0  THEN
				sItemText = SUBSTR(sItemText, 2, LEN(sItemText)-2)
			ELSE
				sItemText = SUBSTR(sItemText, 2, LEN(sItemText)-1)
			ENDIF
		ENDIF
		EXECUTE sGroup+"[\""+sItem+"\"]=\""+DoubleBackslash(sItemText)+"\"\n"
	ENDWHILE

	RETURN True
ENDPROC
